require "MiddleClass.lua"
require "TiledMap.lua"
require "Player.lua"	
require "NPC.lua"
require "Candle.lua"
require "miab/MessageInABottle.lua"

Game = class('Game')

function Game:initialize()
	-- Set the second argument to true to pad
	-- tiles to a "power of 2" width and height
	-- for older graphics cards.
	gr = love.graphics
	gr.height = gr.getHeight( )
	gr.width = gr.getWidth( )
	self.map = TiledMap:new("Map.tmx",false)
	self.player = Player:new()
	
	-- Tracking time of data
	self.hour = 20
	self.minute = 00
	self.minuteOffset = love.timer.getTime()
	
	-- Darkness based on time. (21.5 = 255/12)
	Candle.shroud_intensity = self.hour * 21.5
	
	-- NPC's
	self.NPCs = {}
	table.insert(self.NPCs, NPC:new(self,"character.png", 50,250))
	table.insert(self.NPCs, NPC:new(self,"character.png", 50,450))
	
	-- For dialog windows using MIAB
	self.ocean = Ocean:new()
	love.graphics.setBackgroundColor( 255,255, 255 )
end

function Game:keypressed(key,unicode)
	-- 'z' is our activate key, check objects we could be near
	-- that can be 'activated'
	if key == 'z' then
		-- check npc's
		for _, v in ipairs(self.NPCs) do
			v:checkActivated()
		end
	elseif key == 'q' then
		love.event.push('q')
	end
end

function Game:update(dt)
	if love.keyboard.isDown('m') then
		self.map = TiledMap:new("Map.tmx")
	end
	
	if love.keyboard.isDown('1') then
		self.hour = self.hour + 1
	end
		
	if self.hour >= 24 then
		self.hour = 0
	end
	
	-- Update NPC's
	self.ocean:update(dt)
	for _, v in ipairs(self.NPCs) do
		v:update(dt)
	end
	
	
	--[[ CLOCK BULLSHIT ]]--
	--   TODO: Refactor this and pull it into it's
	--         own class.
    -- 1 in-game minute = 1 irl-second, 1 in-game hour = 60 irl-seconds.
	self.minute = math.floor(love.timer.getTime() - self.minuteOffset) % 61
	if self.minute >= 60 then
		self.hour = self.hour + 1
		self.minuteOffset = love.timer.getTime()
		if self.hour >= 12 and self.hour ~= 24 then
			Candle.shroud_intensity = Candle.shroud_intensity + 21.5
		else
			if self.hour ~= 24 then
				Candle.shroud_intensity = Candle.shroud_intensity - 21.5
			end
		end
	
	end
	
	if self.hour == 12 then
		Candle.shroud_intensity = 0
	end
	--[[ END OF CLOCK BULLSHIT ]]--


	self.map:update(dt)
	self.player:update(dt)
	Candle.update(dt)
end

function Game:draw()
	self.map:draw()
	---[[
	-- Draw NPCs
	for _, v in ipairs(self.NPCs) do
		v:draw()
	end
	

	self.player:draw()
	--]]
	
	--[[ Candle.draw accepts a x/y offset for where the lightsources are being drawn.
	      This does not effect our shroud. We need to offset from players location so
		  that they don't move with the screen.]]
	--Candle.draw(math.floor(self.player.x), math.floor(self.player.y))
	
	self.ocean:draw()
	
	love.graphics.print("x: " .. math.floor(self.player.x) .. "\ny: " .. math.floor(self.player.y), 20, 20)
	self:printTime(20,1024-30)
end

--[[ Turns 1 into 01, keeps 12 as 12. Pads a 0
      to the left of one character strings ]]--
function Game:padTime(t)
	if string.len(t) == 1 then
		return "0" .. t
	end
	
	return t
end

-- Prints the in-game time at x/y
function Game:printTime(x,y)
	local dispHour = 0
	local m = "am"
	if self.hour >= 12 then
		dispHour = self.hour - 12
		if self.hour == 12 then
			dispHour = self.hour
		end

		m = "pm"
	else
		dispHour = self.hour
		if self.hour == 0 then
			dispHour = 12
		end

		m = "am"
	end
	
	love.graphics.print(self:padTime(dispHour) .. ":" .. self:padTime(self.minute) .. " " .. m,x,y)
end
