
require "MiddleClass.lua" -- An OO Library for Lua by Kikito
require "miab/MessageInABottle.lua" -- A dialog library by TechnoCat


NPC = class('NPC')

function NPC:initialize(parent, sprite_src, x, y)
	-- Assign parameters to class members
	self.parent = parent
	self.sprite_src = sprite_src
	self.x = x
	self.y = y
	self.range = 100
	
	-- Load our sprite
	self.sprite = love.graphics.newImage(self.sprite_src)
end

function NPC:checkActivated()
	-- If the player is within 10px then
	-- their activation activates us! 
	if self:inPlayerRange() then
		self:activate()
	end
end

function NPC:activate()
	local msg = "TechnoCat rocks!"
	self.parent.ocean:addBottle(StayBottle:new(nil,msg,nil,self.x+400,self.y))
end

function NPC:inPlayerRange()
	local dist = math.sqrt((self.parent.player.x+400 - self.x)^2+(self.parent.player.y+400-self.y)^2)
	if dist <= self.range then
		local x = self.x - 10
		local y = self.y + 40
		local w = self.sprite:getWidth()+20
		local h = self.sprite:getHeight()-40
		love.graphics.setColor(0,0,200,100)
		love.graphics.rectangle('fill', x-self.parent.player.x,y-self.parent.player.y,w,h)
		love.graphics.setColor(255,255,255)
		love.graphics.print('Press \'Z\' to activate.', x-self.parent.player.x, y - self.parent.player.y + 140) 
		love.graphics.setColor(255,255,255)
		return true
	else
		return false
	end
end

function NPC:update(dt)

end

function NPC:draw(dt)
	local offsetx = self.parent.player.x
	local offsety = self.parent.player.y
	
	love.graphics.draw(self.sprite,self.x - offsetx, self.y - offsety)
	self:inPlayerRange()
end