require "MiddleClass.lua" -- <3 Kikito!
require "dd.lua"

TiledMap = class('TiledMap')

function TiledMap:initialize(filename, tilepadding)
	self.error = nil -- Used to provide errors back
	
	-- Make sure our map-file exists, if not, 
	-- give an error back.
	if love.filesystem.exists(filename) then
		self.filename = filename
	else
		self.filename = nil
		self.error = "File " .. filename .. " does not exist."
	end
	
	if self.filename ~= nil then
		self.raw_data = love.filesystem.read(filename)
	end
	
	-- Remove all Lightsources before we load this map
	-- in-case we've already loaded some before.
	Candle.clearFlames()
	
	-- Hackity hack hack hack!
	self.tileset = {}
	self.tileset.quads = {}   -- We will use this to blit only the tiles
	                          -- that we're going to use on this map.
	self.tiles = {} -- Stores the position of all tiles on the map.
	
	self.layers = 1     -- Total number of layers in our map.
	self.tilecount = 1  -- Total number of possible tiles in any given layer.
	
	self.data = {} -- Generic map data, "name", "width", etc.
	self.doDraw = true
	
	-- Exit rects are squares on the screen that
	-- send our player to another map.
	-- [index] = {x,y,w,h,destx,desty,map_to}
	
	self.exit_rects = {} -- Stores the location of all map exit points.
	self.exit_rect_count = 0 -- Total number of exits on the map.
	
	self.collision_layer = nil
	self.light_layer = nil
	
	-- Used in our parsing, discarded after.
	local inTilesetTag = false
	local inLayerTag = false
	local inObjectTag = false
	
	-- Super ugly hackity-loop that reads
	-- in a .tmx file and parses out the data we need.
	-- XML parsers? For pansies!
	for line in love.filesystem.lines(filename) do
		if line:sub(1,2) == "<m" then
			self:parseMapTag(line)
		elseif line:find("<tileset") ~= nil or inTilesetTag then
			inTilesetTag = true
			if not self:parseTilesetTag(line) then inTilesetTag = false end
		elseif line:find("<layer") ~= nil or inLayerTag then
			inLayerTag = true
			if not self:parseLayerTag(line) then inLayerTag = false end
		elseif line:find("<object") or inObjectTag then
			inObjectTag = true
			if not self:parseObjectTag(line) then inObjectTag = false end
		end
	end
	
	self.layers = self.layers-1 

	--[[ TODO: Change this code to use Objects in the ObjectLayer which gives us
	     control over radiosity and illumination levels within Tiled
	]]
	
	-- Spawn light sources if they exist
	---[[
	if self.light_layer ~= nil then
		for i=1, #self.tiles[self.light_layer] do
			
			if tonumber(self.tiles[self.light_layer][i]) == 17 then
				  Candle.new(math.floor(i%self.data.width * self.data.tilewidth)  - 50,
						     math.floor(i/self.data.width) * self.data.tileheight +50,100,75)
				  Candle.flames[#Candle.flames].light_intensity = 255
			end
		end
	end
	--]]
	
end

function  TiledMap:updateTilesetBatch()
  self.tilesetBatch:clear()
  for l=1, self.layers do
		local i = 1
		for x=1, self.data.width do
			for y=1, self.data.height do
				--[[ Don't draw shit off the screen, afterall, who's going to be there
				     to appreciate that shit!? ITS A WASTE OF CPU CYCLES!!! ]]
				if (y-1)*self.tileset.tilew > game.player.x - 100 and
				   (y-1)*self.tileset.tilew < game.player.x + gr.width and 
				   (x-1)*self.data.tileheight > game.player.y - 150 and
				   (x-1)*self.data.tileheight < game.player.y + gr.height  then
														   
					-- Don't draw if it's the collision/light layer
					if self.collision_layer ~= l and
					   self.light_layer ~= l	 then
						if (tonumber(self.tiles[l][i])) ~= 0  then
							self.tilesetBatch:addq(self.tileset.quads[self.tiles[l][i]],(y-1)*self.tileset.tilew, (x-1)*self.data.tileheight)
							--if not once then print((y-1)*self.tileset.tilew,(x-1)*self.data.tileheight,self.tileset.quads[self.tiles[l][i]].x,self.tileset.quads[self.tiles[l][i]].y) end
						end
					end
				end
				i = i + 1
			end
		end
		--once=true 
  end
end

function TiledMap:draw()
	-- This translation gives the effect that the map follows the player 
	love.graphics.push()
	love.graphics.translate(math.floor(-game.player.x),math.floor(-game.player.y))
	love.graphics.draw(self.tilesetBatch)
	-- Don't translate anything else
	love.graphics.pop()
end

function TiledMap:update(dt)
	Candle.update(dt)
	self:updateTilesetBatch()
end

-- Non-passable tile collision detection and
-- checks if we're entering an exit.
function TiledMap:checkCollision(x,y,w,h)
	-- 1024 768 8 6, 1024 1024 6 6
	local gx={
	math.floor((game.player.x+(gr.width-1024)/2-22)/101)+6,
	--math.floor((game.player.x+(gr.width-1024)/2+8)/101)+6,
	math.floor((game.player.x+(gr.width-1024)/2+38)/101)+6
	}
	local gy = math.floor((game.player.y+(gr.height-1024)/2-16-40)/game.map.data.tileheight)+6
	--print(gx,gy)
	
	local px = math.floor(game.player.x)+400 -- WHY.
	local py = math.floor(game.player.y)+400
	
	-- Let's see if we're colliding with any room exits!
	-- If we are, eff collision, let's load a new map!
	for _, v in ipairs(self.exit_rects) do
		if px >= tonumber(v.x) and px <= tonumber(v.x)+tonumber(v.width) and
		   py >= tonumber(v.y) and py <= tonumber(v.y)+tonumber(v.height) then
			print(":(")
			-- We're in an exit rect, but WHERE TO!?!?
			player.x = v.destx
			player.y = v.desty

			self:initialize(v.to)
		end
	end
	
	-- Are we colliding with a non-passable tile in our collision layer?
	-- If so, then return true so we can tell our player "YOU NO GO!"
	
	--print(colnum,y*self.data.width+x,game.map.tiles[1][y*self.data.width+x],self.data.width)
	for i=1,#gx do
		colnum = tonumber(game.map.tiles[game.map.collision_layer][gy*self.data.width+gx[i]])
		if colnum ~= 0 then return true end
	end
	return false
end

function TiledMap:getError()
	if self.error ~= nil then
		return self.error
	end
	
	return "What errors you fool? :o)"
end

-- [[ MAP PARSING FUNCTIONS ]] --
function TiledMap:parseTilesetTag(line)
	if line:find("<tileset") ~= nil then
		_,_,self.tileset.tilew = line:find("tilewidth=\"(%d*)\"")
		_,_,self.tileset.tileh = line:find("tileheight=\"(%d*)\"")
	end
	
	if line:find("<image") ~= nil then
		_,_,self.tileset.src = line:find("source=\"((%w*).[p][n][g])\"")
		self.tileset.src = love.graphics.newImage(self.tileset.src)
		self.tilesetBatch = love.graphics.newSpriteBatch(self.tileset.src ,self.data.width*self.data.height )
	end
	
	if line:find("</tileset>") ~= nil then
		return false
	end
	
	return true
end

function TiledMap:parseObjectTag(line)
	if line:find("<objectgroup") ~= nil then
		return false
	end
	
	if line:find("<object") ~= nil then
		table.insert(self.exit_rects,{})
		self.exit_rect_count = self.exit_rect_count + 1
		
		_,_,self.exit_rects[self.exit_rect_count].x = line:find("x=\"((%-*)%d*)\"")
		_,_,self.exit_rects[self.exit_rect_count].y = line:find("y=\"((%-*)%d*)\"")
		_,_,self.exit_rects[self.exit_rect_count].width = line:find("width=\"(%d*)\"")
		_,_,self.exit_rects[self.exit_rect_count].height = line:find("height=\"(%d*)\"")
	end

	if line:find("property") ~= nil then
		if line:find("destx") ~= nil then
			_,_,self.exit_rects[self.exit_rect_count].destx = line:find("value=\"((%-*)%d*)\"")
		end
		if line:find("desty") ~= nil then
			_,_,self.exit_rects[self.exit_rect_count].desty = line:find("value=\"((%-*)%d*)\"")
		end
		if line:find("exit") ~= nil then
			_,_,self.exit_rects[self.exit_rect_count].to = line:find("value=\"(%a*%d*)") --"
			self.exit_rects[self.exit_rect_count].to = self.exit_rects[self.exit_rect_count].to .. ".tmx"
		end	
	end
	
	return true
end

function TiledMap:parseLayerTag(line)
	-- Ignore that stupid fucking data tag! WHO CARES ABOUT
	-- STRUCTURE, bwhahaha...		
	
	if line:find("</layer>") ~= nil then
		self.tilecount = self.tilecount + 1
		self.layers = self.layers+1
		self.tilecount = 1
		return false
	end
	

	if line:find("<tile") then
		if self.tilecount == 1 then
			table.insert(self.tiles,{})
		end
		
		_,_,self.tiles[self.layers][self.tilecount] = line:find("gid=\"(%d*)\"")
		--print(self.tilecount)
		-- Create a quad for drawing this tile.
		if self.light_layer == nil and
		   self.collision_layer ~= self.layers then
			if self.tileset.quads[self.tilecount] == nil  and
			   tonumber(self.tiles[self.layers][self.tilecount]) ~= 0 then
			   	local mod=(self.tiles[self.layers][self.tilecount]-1)%10
			   	local flo=math.floor((self.tiles[self.layers][self.tilecount]-1)/10)
				self.tileset.quads[self.tiles[self.layers][self.tilecount]] =  love.graphics.newQuad(mod*self.tileset.tilew, flo*self.tileset.tileh,
																			self.tileset.tilew, self.tileset.tileh,
																			self.tileset.src:getWidth(),
																			self.tileset.src:getHeight())
																			--print(mod,flo)
				if self.tilecount == 1 then print(mod*self.tileset.tilew, flo*self.tileset.tileh) end
				
			end
		end
		self.tilecount = self.tilecount + 1
	end
		if line:find("collision") ~= nil then
		self.collision_layer = self.layers
	end
	
	if line:find("lightsources") ~= nil then
		self.light_layer = self.layers
	end
	--self:updateTilesetBatch()
	return true
end

function TiledMap:parseMapTag(line)
	_,_,self.data.version     = line:find('version=\"(%d*)\"')
	_,_,self.data.orientation = line:find('orientation=\"(%a*)\"')
	_,_,self.data.width = line:find('width=\"(%d*)\"')
	_,_,self.data.height = line:find('height=\"(%d*)\"')
	_,_,self.data.tilewidth = line:find('tilewidth=\"(%d*)\"')	
	_,_,self.data.tileheight = line:find('tileheight=\"(%d*)\"')
end
--[[ END OF MAP PARSING FUNCTIONS ]] --
